/*
 * HND MIPS boards setup routines
 *
 * Copyright (C) 2009, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: setup.c,v 1.72 2010/02/26 04:14:21 Exp $
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/serialP.h>
#include <linux/ide.h>
#include <linux/seq_file.h>
#include <asm/bootinfo.h>
#include <asm/cpu.h>
#include <asm/time.h>
#include <asm/reboot.h>

#ifdef CONFIG_MTD_PARTITIONS
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/romfs_fs.h>
#include <linux/cramfs_fs.h>
#include <linux/squashfs_fs.h>
#endif

#include <typedefs.h>
#include <osl.h>
#include <bcmutils.h>
#include <bcmnvram.h>
#include <siutils.h>
#include <hndcpu.h>
#include <mips33_core.h>
#include <mips74k_core.h>
#include <hndmips.h>
#include <sbchipc.h>
#include <hndchipc.h>
#include <trxhdr.h>
#ifdef CONFIG_BLK_DEV_INITRD
#include <linux/blk.h>
#endif
#ifdef HNDCTF
#include <ctf/hndctf.h>
#endif /* HNDCTF */
#include "bcm947xx.h"

extern void bcm947xx_time_init(void);
extern void bcm947xx_timer_setup(struct irqaction *irq);

#ifdef CONFIG_REMOTE_DEBUG
extern void set_debug_traps(void);
extern void rs_kgdb_hook(struct serial_state *);
extern void breakpoint(void);
#endif

#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)
extern struct ide_ops std_ide_ops;
#endif

/* Enable CPU wait or not*/
extern int cpu_wait_enable;

/* Global assert type */
extern uint32 g_assert_type;

/* Global SB handle */
si_t *bcm947xx_sih = NULL;
spinlock_t bcm947xx_sih_lock = SPIN_LOCK_UNLOCKED;
EXPORT_SYMBOL(bcm947xx_sih);
EXPORT_SYMBOL(bcm947xx_sih_lock);

/* Convenience */
#define sih bcm947xx_sih
#define sih_lock bcm947xx_sih_lock

#ifdef HNDCTF
ctf_t *kcih = NULL;
EXPORT_SYMBOL(kcih);
ctf_attach_t ctf_attach_fn = NULL;
EXPORT_SYMBOL(ctf_attach_fn);
#endif /* HNDCTF */

/* Kernel command line */
char arcs_cmdline[CL_SIZE] __initdata = CONFIG_CMDLINE;
static int lanports_enable = 0;

static void
bcm947xx_reboot_handler(void)
{
	int wombo_reset;

	if (lanports_enable) {
		uint lp = 1 << lanports_enable;

		si_gpioout(sih, lp, 0, GPIO_DRV_PRIORITY);
		si_gpioouten(sih, lp, lp, GPIO_DRV_PRIORITY);
		bcm_mdelay(1);
	}

	/* gpio 0 is also valid wombo_reset */
	if ((wombo_reset = getgpiopin(NULL, "wombo_reset", GPIO_PIN_NOTDEFINED)) !=
	    GPIO_PIN_NOTDEFINED) {
		int reset = 1 << wombo_reset;

		si_gpioout(sih, reset, 0, GPIO_DRV_PRIORITY);
		si_gpioouten(sih, reset, reset, GPIO_DRV_PRIORITY);
		bcm_mdelay(10);
	}
}
void
bcm947xx_machine_restart(char *command)
{
	printk("Please stand by while rebooting the system...\n");

	/* Set the watchdog timer to reset immediately */
	__cli();
	bcm947xx_reboot_handler();
	hnd_cpu_reset(sih);
}

void
bcm947xx_machine_halt(void)
{
	printk("System halted\n");

	/* Disable interrupts and watchdog and spin forever */
	__cli();
	si_watchdog(sih, 0);
	bcm947xx_reboot_handler();
	while (1);
}

#ifdef CONFIG_SERIAL

static struct serial_struct rs = {
	line: 0,
	flags: ASYNC_BOOT_AUTOCONF,
	io_type: SERIAL_IO_MEM,
};

static void __init
serial_add(void *regs, uint irq, uint baud_base, uint reg_shift)
{
	rs.iomem_base = regs;
	rs.irq = irq + 2;
	rs.baud_base = baud_base / 16;
	rs.iomem_reg_shift = reg_shift;

	early_serial_setup(&rs);

	rs.line++;
}

static void __init
serial_setup(si_t *sih)
{
	si_serial_init(sih, serial_add);

#ifdef CONFIG_REMOTE_DEBUG
	/* Use the last port for kernel debugging */
	if (rs.iomem_base)
		rs_kgdb_hook(&rs);
#endif
}

#endif /* CONFIG_SERIAL */

void __init
brcm_setup(void)
{
	char *value;

	/* Get global SI handle */
	sih = si_kattach(SI_OSH);

	/* Initialize clocks and interrupts */
	si_mips_init(sih, SBMIPS_VIRTIRQ_BASE);

	if (BCM330X(mips_cpu.processor_id) &&
		(read_c0_diag() & BRCM_PFC_AVAIL)) {
		/* 
		 * Now that the sih is inited set the proper PFC value 
		 */	
		printk("Setting the PFC to its default value\n");
		enable_pfc(PFC_AUTO);
	}


#ifdef CONFIG_SERIAL
	/* Initialize UARTs */
	serial_setup(sih);
#endif

#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)
	ide_ops = &std_ide_ops;
#endif

	value = nvram_get("assert_type");
	if (value && strlen(value))
		g_assert_type = simple_strtoul(value, NULL, 10);

	/* Override default command line arguments */
	value = nvram_get("kernel_args");
	if (value && strlen(value) && strncmp(value, "empty", 5))
		strncpy(arcs_cmdline, value, sizeof(arcs_cmdline));


	if ((lanports_enable = getgpiopin(NULL, "lanports_enable", GPIO_PIN_NOTDEFINED)) ==
		GPIO_PIN_NOTDEFINED)
		lanports_enable = 0;
	/* Generic setup */
	_machine_restart = bcm947xx_machine_restart;
	_machine_halt = bcm947xx_machine_halt;
	_machine_power_off = bcm947xx_machine_halt;

	board_time_init = bcm947xx_time_init;
	board_timer_setup = bcm947xx_timer_setup;

	/* Check if we want to enable cpu wait */
	if(nvram_match("wait", "1")) 
		cpu_wait_enable = 1;
}

const char *
get_system_type(void)
{
	static char s[32];
	char cn[8];

	if (bcm947xx_sih) {
		bcm_chipname(bcm947xx_sih->chip, cn, 8);
		sprintf(s, "Broadcom BCM%s chip rev %d",
		        cn, bcm947xx_sih->chiprev);
		return s;
	} else
		return "Broadcom BCM947XX";
}

void
bcm947xx_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "System clocks\n  (cpu/mem/si/xtal)\t: %u/%u/%u/%u Mhz.\n",
		   si_cpu_clock(sih) / 1000000,
		   si_mem_clock(sih) / 1000000,
		   si_clock(sih) / 1000000,
		   si_alp_clock(sih) / 1000000);
}
void __init
bus_error_init(void)
{
}

#ifdef CONFIG_MTD_PARTITIONS

static struct mtd_partition bcm947xx_parts[] = {
#if  defined(ENVRAM)
	/* We want flash boot partition to be wirtable only in the internal images
	and only in linux-internal-router not in any other images */
	{ name: "boot", offset: 0, size: 0, },
#else
	{ name: "boot",	offset: 0, size: 0, mask_flags: MTD_WRITEABLE, },
#endif 
	{ name: "linux", offset: 0, size: 0, },
	{ name: "rootfs", offset: 0, size: 0, mask_flags: MTD_WRITEABLE, },
    
#ifdef X_ST_ML
    { name: "ML1", offset: 0, size: 0, },
	{ name: "ML2", offset: 0, size: 0, },
	{ name: "ML3", offset: 0, size: 0, },
	{ name: "ML4", offset: 0, size: 0, },
	{ name: "ML5", offset: 0, size: 0, },
	{ name: "ML6", offset: 0, size: 0, },
	{ name: "ML7", offset: 0, size: 0, },
#else
    /* reserve 2 sectors for multi-language implementation */
	{ name: "ML1", offset: 0, size: 0, },
	{ name: "ML2", offset: 0, size: 0, },
#endif
    
    
	{ name: "T_Meter1", offset: 0, size: 0, },
	{ name: "T_Meter2", offset: 0, size: 0, },
        
#ifdef HSDPA
        { name: "T_Meter3", offset: 0, size: 0, },
        { name: "T_Meter4", offset: 0, size: 0, },
#endif
        
	{ name: "POT", offset: 0, size: 0, },
	{ name: "board_data", offset: 0, size: 0, },
    
	{ name: "nvram", offset: 0, size: 0, },
	{ name: NULL, },
};

struct mtd_partition * __init
init_mtd_partitions(struct mtd_info *mtd, size_t size)
{
	struct romfs_super_block *romfsb;
	struct cramfs_super *cramfsb;
	struct squashfs_super_block *squashfsb;
	struct trx_header *trx;
	unsigned char buf[512];
	int off;
	size_t len;
	int i;

	romfsb = (struct romfs_super_block *) buf;
	cramfsb = (struct cramfs_super *) buf;
	squashfsb = (struct squashfs_super_block *) buf;
	trx = (struct trx_header *) buf;

	/* Look at every 64 KB boundary */
	for (off = 0; off < size; off += (64 * 1024)) {
		memset(buf, 0xe5, sizeof(buf));

		/*
		 * Read block 0 to test for romfs and cramfs superblock
		 */
		if (MTD_READ(mtd, off, sizeof(buf), &len, buf) ||
		    len != sizeof(buf))
			continue;

		/* Try looking at TRX header for rootfs offset */
		if (le32_to_cpu(trx->magic) == TRX_MAGIC) {
			bcm947xx_parts[1].offset = off;
			if (le32_to_cpu(trx->offsets[1]) > off)
				off = le32_to_cpu(trx->offsets[1]);
			/* In case where CFE boots from ROM, we expect
			 * Linux to fit in first flash partition.
			 */
			if (bcm947xx_parts[1].offset == 0 && off)
				off -= (64 * 1024);
			continue;
		}

		/* romfs is at block zero too */
		if (romfsb->word0 == ROMSB_WORD0 &&
		    romfsb->word1 == ROMSB_WORD1) {
			printk(KERN_NOTICE
			       "%s: romfs filesystem found at block %d\n",
			       mtd->name, off / BLOCK_SIZE);
			goto done;
		}

		/* so is cramfs */
		if (cramfsb->magic == CRAMFS_MAGIC) {
			printk(KERN_NOTICE
			       "%s: cramfs filesystem found at block %d\n",
			       mtd->name, off / BLOCK_SIZE);
			goto done;
		}
        
		if(squashfsb->s_magic == SQUASHFS_MAGIC)	{
			printk(KERN_NOTICE
			       "%s: squashfs filesystem found at block %d\n",
			       mtd->name, off / BLOCK_SIZE);
			goto done;
		}
		

		if (squashfsb->s_magic == SQUASHFS_MAGIC_LZMA) {
			printk(KERN_NOTICE
			       "%s: squash filesystem with lzma found at block %d\n",
			       mtd->name, off / BLOCK_SIZE);
			goto done;
		}
	}

	printk(KERN_NOTICE
	       "%s: Couldn't find valid ROM disk image\n",
	       mtd->name);

 done:
    
#ifdef X_ST_ML
    bcm947xx_parts[14].offset = size - ROUNDUP(NVRAM_SPACE, mtd->erasesize);
	bcm947xx_parts[14].size = size - bcm947xx_parts[14].offset;
	
    /* Add Board data and POT partition */
	bcm947xx_parts[13].offset = bcm947xx_parts[14].offset - mtd->erasesize;
	bcm947xx_parts[13].size = mtd->erasesize;

	bcm947xx_parts[12].offset = bcm947xx_parts[13].offset - mtd->erasesize;
	bcm947xx_parts[12].size = mtd->erasesize;

    /* Add traffic meter partition */
	bcm947xx_parts[11].offset = bcm947xx_parts[12].offset - mtd->erasesize;
	bcm947xx_parts[11].size = mtd->erasesize;

	bcm947xx_parts[10].offset = bcm947xx_parts[11].offset - mtd->erasesize;
	bcm947xx_parts[10].size = mtd->erasesize;    

    /* Add multi-language partitions */    
    bcm947xx_parts[9].offset = bcm947xx_parts[10].offset - mtd->erasesize;
	bcm947xx_parts[9].size = mtd->erasesize;
	
    bcm947xx_parts[8].offset = bcm947xx_parts[9].offset - mtd->erasesize;
	bcm947xx_parts[8].size = mtd->erasesize;
    
    bcm947xx_parts[7].offset = bcm947xx_parts[8].offset - mtd->erasesize;
	bcm947xx_parts[7].size = mtd->erasesize;
    
    bcm947xx_parts[6].offset = bcm947xx_parts[7].offset - mtd->erasesize;
	bcm947xx_parts[6].size = mtd->erasesize;
    
    bcm947xx_parts[5].offset = bcm947xx_parts[6].offset - mtd->erasesize;
	bcm947xx_parts[5].size = mtd->erasesize;
    
	bcm947xx_parts[4].offset = bcm947xx_parts[5].offset - mtd->erasesize;
	bcm947xx_parts[4].size = mtd->erasesize;

	bcm947xx_parts[3].offset = bcm947xx_parts[4].offset - mtd->erasesize;
	bcm947xx_parts[3].size = mtd->erasesize;
#else
    
	/* Find and size nvram */
#ifdef HSDPA 
	bcm947xx_parts[11].offset = size - ROUNDUP(NVRAM_SPACE, mtd->erasesize);
	bcm947xx_parts[11].size = size - bcm947xx_parts[11].offset;

    /* Add traffic meter partition */
	bcm947xx_parts[10].offset = bcm947xx_parts[11].offset - mtd->erasesize;
	bcm947xx_parts[10].size = mtd->erasesize;

	bcm947xx_parts[9].offset = bcm947xx_parts[10].offset - mtd->erasesize;
	bcm947xx_parts[9].size = mtd->erasesize;
#else
	bcm947xx_parts[9].offset = size - ROUNDUP(NVRAM_SPACE, mtd->erasesize);
	bcm947xx_parts[9].size = size - bcm947xx_parts[9].offset;
#endif
    /* Add Board data and POT partition */
	bcm947xx_parts[8].offset = bcm947xx_parts[9].offset - mtd->erasesize;
	bcm947xx_parts[8].size = mtd->erasesize;

	bcm947xx_parts[7].offset = bcm947xx_parts[8].offset - mtd->erasesize;
	bcm947xx_parts[7].size = mtd->erasesize;

    /* Add traffic meter partition */
	bcm947xx_parts[6].offset = bcm947xx_parts[7].offset - mtd->erasesize;
	bcm947xx_parts[6].size = mtd->erasesize;

	bcm947xx_parts[5].offset = bcm947xx_parts[6].offset - mtd->erasesize;
	bcm947xx_parts[5].size = mtd->erasesize;    

    /* Add multi-language partitions */
	bcm947xx_parts[4].offset = bcm947xx_parts[5].offset - mtd->erasesize;
	bcm947xx_parts[4].size = mtd->erasesize;

	bcm947xx_parts[3].offset = bcm947xx_parts[4].offset - mtd->erasesize;
	bcm947xx_parts[3].size = mtd->erasesize;    
    
#endif
    
	/* Find and size rootfs */
	if (off < size) {
		bcm947xx_parts[2].offset = off;
		bcm947xx_parts[2].size = bcm947xx_parts[3].offset - bcm947xx_parts[2].offset;
	}

	/* Size linux (kernel and rootfs) */
	bcm947xx_parts[1].size = bcm947xx_parts[3].offset - bcm947xx_parts[1].offset;

	/* Size pmon */
	bcm947xx_parts[0].size = bcm947xx_parts[1].offset - bcm947xx_parts[0].offset;

	return bcm947xx_parts;
}

EXPORT_SYMBOL(init_mtd_partitions);

#ifdef CONFIG_BLK_DEV_INITRD
extern void * __rd_start, * __rd_end;
extern char _end;
static __init int
check_ramdisk_trx(unsigned long offset, unsigned long ram_size)
{
	struct trx_header *trx;
	uint32 crc;
	unsigned int len;
	uint8 *ptr = (uint8 *)offset;

	trx = (struct trx_header *)ptr;

	/* Not a TRX_MAGIC */
	if (le32_to_cpu(trx->magic) != TRX_MAGIC) {
		printk("check_ramdisk_trx: not a valid TRX magic\n");
		return -1;
	}

	/* TRX len invalid */
	len = le32_to_cpu(trx->len);
	if (offset + len > ram_size) {
		printk("check_ramdisk_trx: not a valid TRX length\n");
		return -1;
	}

	/* Checksum over header */
	crc = hndcrc32((uint8 *) &trx->flag_version,
		sizeof(struct trx_header) - OFFSETOF(struct trx_header, flag_version),
		CRC32_INIT_VALUE);

	/* Move ptr to data */
	ptr += sizeof(struct trx_header);
	len -= sizeof(struct trx_header);

	/* Checksum over data */
	crc = hndcrc32(ptr, len, crc);

	/* Verify checksum */
	if (le32_to_cpu(trx->crc32) != crc) {
		printk("check_ramdisk_trx: checksum invalid\n");
		return -1;
	}

	return 0;
}

void __init init_ramdisk(unsigned long mem_end)
{
	struct trx_header *trx = NULL;
	char *from_rootfs, *to_rootfs;
	unsigned long rootfs_size = 0;
	unsigned long ram_size = mem_end + 0x80000000;
	unsigned long offset;
	char *root_cmd = "root=/dev/ram0 ramdisk_size=4096 ramdisk_blocksize=4096 console=ttyS0,115200";

	/* embedded ramdisk */
	if (&__rd_start != &__rd_end)
		return;

	to_rootfs = (((unsigned long)&_end + PAGE_SIZE-1) & PAGE_MASK);
	offset = ((unsigned long)&_end +0xffff) & ~0xffff;

	/* Look at TRX header from end of linux */
	for (; offset < ram_size; offset += 0x10000) {
		trx = (struct trx_header *)offset;
		if (le32_to_cpu(trx->magic) == TRX_MAGIC &&
			check_ramdisk_trx(offset, ram_size) == 0) {
			printk(KERN_NOTICE
				   "Found TRX image  at %08lx\n", offset);
			from_rootfs = (char *)((unsigned long)trx + le32_to_cpu(trx->offsets[1]));
			rootfs_size = le32_to_cpu(trx->len) - le32_to_cpu(trx->offsets[1]);
			rootfs_size = (rootfs_size + 0xffff) & ~0xffff;
			printk("rootfs size is %ld bytes at 0x%p, copying to 0x%p\n",
				rootfs_size, from_rootfs, to_rootfs);
			memmove(to_rootfs, from_rootfs, rootfs_size);

			initrd_start = (int)to_rootfs;
			initrd_end = initrd_start + rootfs_size;
			strncpy(arcs_cmdline, root_cmd, sizeof(arcs_cmdline));
			/* 
			 * In case the system warm boot, the memory won't be zeroed.
			 * So we have to erase trx magic.
			 */
			if (initrd_end < (unsigned long)trx)
				trx->magic = 0;
			break;
		}
	}
}
#endif /* CONFIG_BLK_DEV_INITRD */
#endif
